import React from 'react';
import { CompositeLayer, HeatmapLayer } from 'deck.gl';
import * as d3 from 'd3';
import { getDistance } from '../../utlis';
import { CompositeLayerType, Edge, LayerProps } from '../../mapvis.types';
import { Node } from '@graphpolaris/shared/lib/data-access';

export class HeatLayer extends CompositeLayer<CompositeLayerType> {
  static type = 'Heatmap';

  updateState({ changeFlags }: { changeFlags: any }) {
    return changeFlags.propsOrDataChanged || changeFlags.somethingChanged;
  }

  createSegments(edges: Edge[]) {
    // Generates a path between source and target nodes
    return edges.map((edge: Edge, index) => {
      const length = getDistance(edge.path[0], edge.path[1]);
      const nSegments = length * this.props.settings.nSegments;

      let xscale = d3
        .scaleLinear()
        .domain([0, nSegments + 1])
        .range([edge.path[0][0], edge.path[1][0]]);

      let yscale = d3
        .scaleLinear()
        .domain([0, nSegments + 1])
        .range([edge.path[0][1], edge.path[1][1]]);

      let source = edge.path[0];
      let target = null;
      let local = [source];

      for (let j = 1; j <= nSegments; j++) {
        target = [xscale(j), yscale(j)];
        local.push(target);
        source = target;
      }

      local.push(edge.path.slice(-1)[0]);
      return { ...edge, path: local };
    });
  }

  renderLayers() {
    const { data, settings, getNodeLocation, selected, setLayerIds, graphMetadata } = this.props;

    const layers: any[] = [];
    const layerIds: string[] = [];

    graphMetadata.nodes.labels.forEach((label: string) => {
      const layerId = `${label}-nodes-heatmaplayer`;
      layerIds.push(layerId);

      console.log(settings.nodes[label].size);
      layers.push(
        new HeatmapLayer<Node>({
          id: layerId,
          data: data.nodes.filter((node: Node) => node.label === label),
          visible: !settings.nodes[label].hidden,
          getPosition: (d: any) => getNodeLocation(d.id),
          getWeight: (d: any) => settings.nodes[label].size,
          radiusPixels: settings.nodes[label].size,
          aggregation: 'SUM',
        }),
      );
    });

    setLayerIds(layerIds);

    return layers;
  }
}

HeatLayer.layerName = 'Heatmap';
