import React from 'react';
import { CompositeLayer } from 'deck.gl';
import { IconLayer } from '@deck.gl/layers';
import { CompositeLayerType, LayerProps } from '../../mapvis.types';
import { Node } from '@graphpolaris/shared/lib/data-access';

const ICON_MAPPING = {
  marker: { x: 0, y: 0, width: 128, height: 128, mask: false },
};

export class NodeIconLayer extends CompositeLayer<CompositeLayerType> {
  static type = 'Icon';

  updateState({ changeFlags }: { changeFlags: any }) {
    return changeFlags.propsOrDataChanged || changeFlags.somethingChanged;
  }

  renderLayers() {
    const { data, settings, getNodeLocation, setLayerIds, graphMetadata } = this.props;

    const layers: any[] = [];
    const layerIds: string[] = [];

    graphMetadata.nodes.labels.forEach((label: string) => {
      const layerId = `${label}-nodes-iconlayer`;
      layerIds.push(layerId);

      layers.push(
        new IconLayer({
          id: layerId,
          data: data.nodes.filter((node: Node) => node.label === label),
          visible: !settings.nodes[label].hidden,
          iconAtlas: 'https://raw.githubusercontent.com/visgl/deck.gl-data/master/website/icon-atlas.png',
          iconMapping: ICON_MAPPING,
          sizeScale: 10,
          pickable: true,
          getIcon: (d: any) => 'marker',
          getColor: (d: any) => settings.nodes[label].color,
          getPosition: (d: any) => getNodeLocation(d._id),
          getSize: (d: any) => 3,
        }),
      );
    });

    setLayerIds(layerIds);

    return layers;
  }
}

NodeIconLayer.layerName = 'Icon';
